"use client";
import { useMemo } from "react";
import Pagination from "./Pagination";
import classNames from "classnames";
var Grid = function (_a) {
    var rows = _a.rows, cols = _a.cols, pagination = _a.pagination;
    var currentPageRows = useMemo(function () {
        if (pagination === undefined) {
            return rows;
        }
        var pageSizeOptions = pagination.pageSizeOptions, pageSizeIndex = pagination.pageSizeIndex, currentPage = pagination.currentPage;
        var pageSize = pageSizeOptions[pageSizeIndex];
        var lowerIndex = pageSize * (currentPage - 1);
        var upperIndex = lowerIndex + pageSize;
        return rows.slice(lowerIndex, upperIndex);
    }, [rows, pagination]);
    var displayRows = useMemo(function () {
        var nameToIndex = new Map();
        cols.forEach(function (_a, index) {
            var name = _a.name;
            nameToIndex.set(name, index);
        });
        return currentPageRows.map(function (row, index) {
            cols
                .map(function (_a) {
                var name = _a.name;
                return name;
            })
                .forEach(function (name) {
                if (!(name in row)) {
                    throw new Error("Column definition specifies a property named \"".concat(name, "\",\n              but it was no found in the row data object at index ").concat(index));
                }
            });
            var displayRow = [];
            Object.keys(row).forEach(function (name) {
                if (!nameToIndex.has(name)) {
                    throw new Error("Row data contains a property named \"".concat(name, "\",\n          but it was not found among the column definitions"));
                }
                var index = nameToIndex.get(name);
                var formatter = cols[index].formatter;
                var typeString = cols[index].type;
                var value = row[name];
                if (formatter) {
                    displayRow[index] = formatter(value);
                    return;
                }
                if (typeString === "date") {
                    displayRow[index] = value.toDateString();
                    return;
                }
                if (typeString === "datetime") {
                    displayRow[index] = value.toLocaleString();
                    return;
                }
                if (typeString === "number") {
                    displayRow[index] = value.toLocaleString();
                    return;
                }
                displayRow[index] = value;
            });
            return displayRow;
        });
    }, [currentPageRows, cols]);
    var handleSetPageNum = function (pageNum) {
        if (pagination === undefined) {
            return;
        }
        pagination.setCurrentPage(pageNum);
    };
    var handleSetPageSize = function (event) {
        if (pagination === undefined) {
            return;
        }
        pagination.setPageSizeIndex(Number(event.target.value));
    };
    // Once this component implements selection state, and if such interactivity is enabled, (conditionally) change the
    // aria role to "grid".
    // Array index is okay for the key for rows until some type of feature involving changing the index of rows, such as
    // sorting or pagination, is implemented.
    return (<div>
      <table className="table">
        <thead>
          <tr>
            {cols.map(function (_a) {
            var name = _a.name, label = _a.label;
            return (<th key={name}>{label}</th>);
        })}
          </tr>
        </thead>
        <tbody>
          {displayRows.map(function (row, index) { return (<tr key={index}>
              {row.map(function (value, index) { return (<td key={index}>{value}</td>); })}
            </tr>); })}
        </tbody>
      </table>
      {pagination && (<div className="d-flex justify-content-end gap-2">
          <div>
            <select className={classNames({
                "form-select": true,
                "form-select-lg": pagination.componentSize === "large",
                "form-select-sm": pagination.componentSize === "small",
            })} value={pagination.pageSizeIndex} aria-label="Number of Rows per Page" onChange={handleSetPageSize}>
              {pagination.pageSizeOptions.map(function (numRows, index) { return (<option key={index} value={index}>
                  {numRows}
                </option>); })}
            </select>
          </div>
          <Pagination numPages={Math.ceil(rows.length /
                pagination.pageSizeOptions[pagination.pageSizeIndex])} pageNum={pagination.currentPage} numButtons={pagination.maxPageButtons} setPageNum={handleSetPageNum} size={pagination.componentSize || "medium"}/>
        </div>)}
    </div>);
};
export default Grid;
